#include "game.h"
#include "building.h"
#include "fusebox.h"
#include "guy.h"
#include "level.h"
#include "particle.h"
#include "font.h"
#include "sound.h"

#define GAME_PLAY	(0)
#define GAME_WIN	(1)
#define GAME_LOSE	(2)

static KGraphic *cursorPic[2],*backgd,*moonPic;
static int msx,msy;
byte curState,fuseFrom,mode;

byte curLevel;
word zombieClock;
word ticker,tallyCount;

void Game_Init(void)
{
	cursorPic[0]=new KGraphic;
	cursorPic[0]->loadPicture(KMiscTools::makeFilePath("gfx\\cursor.png"));
	cursorPic[1]=new KGraphic;
	cursorPic[1]->loadPicture(KMiscTools::makeFilePath("gfx\\cursor2.png"));
	backgd=new KGraphic;
	backgd->loadPicture(KMiscTools::makeFilePath("gfx\\backgd.png"));
	moonPic=new KGraphic;
	moonPic->loadPicture(KMiscTools::makeFilePath("gfx\\moon.png"));

	curState=0;
	zombieClock=0;
	mode=GAME_PLAY;

	Guys_Init();
	Building_Init();
	FuseBox_Init();
	Particle_Init();
	Font_Init();

	ticker=levelDef[curLevel].time;
	tallyCount=0;
}


void Game_Exit(void)
{
	Particle_Exit();
	Guys_Exit();
	Building_Exit();
	FuseBox_Exit();
	Font_Exit();

	delete cursorPic[0];
	delete cursorPic[1];
	delete backgd;
	delete moonPic;
}

byte Game_Update(void)
{
	msx=KInput::getMouseX();
	msy=KInput::getMouseY();

	switch(mode)
	{
		case GAME_PLAY:
			if(KInput::isPressed(K_VK_ESCAPE))
				return 0;

			Guys_Update();
			Particle_Update();

			if(ticker>0)
				ticker--;
			else
			{
				if(levelDef[curLevel].goal==GOAL_SAVE)
					mode=GAME_WIN;
				else
					mode=GAME_LOSE;
			}

			zombieClock++;
			if(zombieClock>=levelDef[curLevel].zombieRate)
			{
				CallZombie();
				zombieClock=0;
			}

			if(curState==0)	// haven't got anything
			{
				if(KInput::getLeftButtonState())
				{
					byte b=FuseClicked(msx,msy);
					if(b!=255 && fuseBox.state[b]==FUSE_ON)
					{
						curState=1;
						fuseBox.state[b]=FUSE_OFF;
						bldg.lit[b]=0;
						fuseFrom=b;
						Sound_Play(SND_FUSEGET);
						// affect people in the space
					}
				}
			}
			else	// holding a fuse
			{
				if(!KInput::getLeftButtonState())	// let go of it
				{
					byte b=FuseClicked(msx,msy);
					if(b==255 || fuseBox.state[b]!=FUSE_OFF)	// can't put it here
					{
						// so drop it back where you got it
						curState=0;
						fuseBox.state[fuseFrom]=FUSE_ON;
						bldg.lit[fuseFrom]=1;
						SparkFuse(fuseFrom);
						Sound_Play(SND_FUSEPUT);
						// affect people in the space
					}
					else
					{
						curState=0;
						fuseBox.state[b]=FUSE_ON;
						bldg.lit[b]=1;
						SparkFuse(b);
						Sound_Play(SND_FUSEPUT);
						// affect people in the space
					}
				}
				
			}
			if(levelDef[curLevel].goal==GOAL_KILL && zombiesSlain>=levelDef[curLevel].goalVal)
				mode=GAME_WIN;
			if(levelDef[curLevel].goal==GOAL_SAVE && peopleCount<levelDef[curLevel].goalVal)
				mode=GAME_LOSE;
			break;
		case GAME_WIN:
			Particle_Update();
			tallyCount++;
			if(tallyCount==10)
				Sound_Play(SND_WIN);
			if(tallyCount==60*5)
				return 2;
			if(KInput::getLeftButtonState() && tallyCount>60*2)
				return 2;
			break;
		case GAME_LOSE:
			Particle_Update();
			tallyCount++;
			if(tallyCount==60)
				Sound_Play(SND_LOSE);
			if(tallyCount==60*5)
				return 0;
			if(KInput::getLeftButtonState() && tallyCount>60*1)
				return 0;
			break;
	}
	return 1;
}

void HUD(void)
{
	char s[32];

	if(levelDef[curLevel].goal==GOAL_SAVE)
	{
		sprintf(s,"SAVE %d!!",levelDef[curLevel].goalVal);
		Print(5,574-24,s);
	}
	sprintf(s,"PEOPLE: %d/%d",peopleCount,peopleStart);
	Print(5,574,s);

	if(levelDef[curLevel].goal==GOAL_KILL)
	{
		sprintf(s,"TORCH %d MORE!",levelDef[curLevel].goalVal-zombiesSlain);
		Print(250,574-24,s);
	}
	sprintf(s,"CHUDS: %d",zombieCount-zombiesSlain);
	Print(325,574,s);
}

void Game_Render(void)
{
	//glClear(GL_COLOR_BUFFER_BIT);
	KGraphic::setDefaultWorldView();
	backgd->blitRect(0,0,800,600,0,0,false,false);

	moonPic->blitAlphaRect(0,0,96,96,700,600-174-(ticker/10),false,false);

	Building_Render();
	
	FuseBox_Render();
	Particle_Render();
	HUD();

	if(mode==GAME_WIN)
	{
		Print(244+(rand()%3),200+(rand()%3),"YOU DID IT!!!");
	}
	else if(mode==GAME_LOSE)
	{
		Print(64+(rand()%3),200+(rand()%3),"THE CHUDS HAVE BESTED YOU!!!");
	}

	Print(800-strlen(levelDef[curLevel].name)*24,1,levelDef[curLevel].name);

	cursorPic[curState]->blitAlphaRect(0,0,23,45,msx-12,msy-8,false,false);
	KWindow::flipBackBuffer(true);
}

byte Game_Run(void)
{
	dword now,start,interval,runs;
	byte retval;
	dword i;

	Game_Init();

	start=KMiscTools::getMilliseconds();

	interval=0;
	retval=1;
	while(1)
	{
		now=KMiscTools::getMilliseconds();
		interval+=now-start;
		start=now;
		if(interval>TIME_PER_FRAME*3)
			interval=TIME_PER_FRAME*3;
		runs=0;
		while(interval>=TIME_PER_FRAME)
		{
			runs++;
			interval-=TIME_PER_FRAME;
		}

		for(i=0;i<runs;i++)
		{
			retval=Game_Update();
			if(retval==0)
				i=runs;
		}
		if(retval==0 || retval==2)
			break;
		Game_Render();
	}
	Game_Exit();
	return retval;
}