#include "guy.h"
#include "building.h"
#include "level.h"
#include "game.h"
#include <math.h>
#include "particle.h"
#include "sound.h"

KGraphic *guyPix;
byte anim;
guy_t guys[MAX_GUYS];
byte zombieCount,peopleCount,zombiesSlain,peopleStart;

void Guys_Init(void)
{
	int i;

	guyPix=new KGraphic;
	guyPix->loadPicture(KMiscTools::makeFilePath("gfx\\guys.png"));

	for(i=0;i<MAX_GUYS;i++)
		guys[i].type=GUY_NONE;
	zombieCount=0;
	zombiesSlain=0;
	peopleCount=0;
	peopleStart=0;
}

void Guys_Exit(void)
{
	delete guyPix;
}

void Guys_Add(byte cell,byte type)
{
	int i;

	for(i=0;i<MAX_GUYS;i++)
	{
		if(guys[i].type==GUY_NONE)
		{
			if(type==GUY_KINGCHUD)
			{
				type=GUY_ZOMBIE;
				guys[i].king=1;
			}
			else
				guys[i].king=0;

			if(type==GUY_GREEN)
			{
				peopleStart++;
				peopleCount++;
			}
			else if(type==GUY_ZOMBIE)
				zombieCount++;
			guys[i].type=type;
			guys[i].cell=cell;
			guys[i].toCell=cell;
			guys[i].state=GS_THINKING;
			guys[i].anim=rand()%6;
			guys[i].animDelay=rand()%10;
			guys[i].dx=0;
			guys[i].dy=0;
			guys[i].x=(cell%BLDG_WIDTH)*BLDG_TILEX+BLDG_X+42.5f;
			guys[i].y=(cell/BLDG_WIDTH)*BLDG_TILEY+BLDG_Y+80.0f;
			guys[i].stepy=0;
			guys[i].stepdy=0;
			guys[i].thinkDelay=60;
			guys[i].life=MAX_LIFE;
			return;
		}
	}
}

void CallZombie(void)
{
	byte c;

	if(zombieCount>=levelDef[curLevel].zombies)
		return;

	while(1)
	{
		c=rand()%BLDG_WIDTH;
		if(bldg.type[c+(BLDG_HEIGHT-1)*BLDG_WIDTH]!=BLDG_EMPTY)
		{
			Guys_Add(BLDG_HEIGHT*BLDG_WIDTH+c,GUY_ZOMBIE);
			return;
		}
	}
}

byte CanEnter(byte type,byte cell,byte fromCell)
{
	int i;

	if(cell>=BLDG_WIDTH*(BLDG_HEIGHT+1))	// allow one more row than the actual height, for the sewers
		return 0;

	if(bldg.type[cell]==BLDG_EMPTY || bldg.type[cell]==BLDG_BRICK)
		return 0;	// no go

	if(type==GUY_GREEN && bldg.type[cell]==BLDG_DOOR)
		return 0;

	if(type==GUY_GREEN && bldg.lit[cell]==0)
		return 0;

	if(type==GUY_ZOMBIE && bldg.lit[cell]==1)
		return 0;

	if(cell>=BLDG_WIDTH*BLDG_HEIGHT)	// in the sewers
	{
		if(bldg.type[cell-BLDG_WIDTH]==BLDG_EMPTY)
			return 0;	// can't enter a sewer space with no building above it
		if(fromCell<BLDG_WIDTH*BLDG_HEIGHT)
			return 0;	// can't re-enter sewers
	}

	for(i=0;i<MAX_GUYS;i++)
	{
		if(guys[i].type==type && guys[i].toCell==cell)
			return 0;	// can't head to a cell someone else of my flavor is occupying/going to
	}
	if(type==GUY_ZOMBIE)	// look for fresh meat
	{
		for(i=0;i<MAX_GUYS;i++)
			if(guys[i].type==GUY_GREEN && guys[i].cell==cell)
				return 2;	// mmm, preferred!
	}

	return 1;
}

void ChoosePlaceToGo(guy_t *me)
{
	byte ok[4];
	byte x,y,tries;

	x=me->cell%BLDG_WIDTH;
	y=me->cell/BLDG_WIDTH;

	if(x<BLDG_WIDTH-1)
		ok[0]=CanEnter(me->type,me->cell+1,me->cell);
	else
		ok[0]=0;
	if(x>0)
		ok[2]=CanEnter(me->type,me->cell-1,me->cell);
	else
		ok[2]=0;
	if(y<BLDG_HEIGHT)
		ok[1]=CanEnter(me->type,me->cell+BLDG_WIDTH,me->cell);
	else
		ok[1]=0;
	if(y>0)
		ok[3]=CanEnter(me->type,me->cell-BLDG_WIDTH,me->cell);
	else
		ok[3]=0;

	if(me->type==GUY_ZOMBIE)
	{

		if(ok[0]<2 && ok[1]<2 && ok[2]<2 && ok[3]<2)
			x=rand()%4;
		else
		{
			x=rand()%4;
			while(ok[x]<2)
			{
				x++;
				if(x>3)
					x=0;
			}
		}
	}
	else
		x=rand()%4;
	tries=0;
	while(tries<4)
	{
		if(ok[x])
		{
			switch(x)
			{
				case 0:
					me->dx=1;
					me->dy=0;
					me->toCell=me->cell+1;
					break;
				case 1:
					me->dx=0;
					me->dy=1;
					me->toCell=me->cell+BLDG_WIDTH;
					break;
				case 2:
					me->dx=-1;
					me->dy=0;
					me->toCell=me->cell-1;
					break;
				case 3:
					me->dx=0;
					me->dy=-1;
					me->toCell=me->cell-BLDG_WIDTH;
					break;
			}
			me->state=GS_MOVING;
			me->stepdy=-2;
			me->stepy=0;
			break;
		}
		else
		{
			x++;
			if(x>3)
				x=0;
			tries++;
		}
	}
}

byte EatGuy(guy_t *me)
{
	int i;
	
	for(i=0;i<MAX_GUYS;i++)
	{
		if(guys[i].cell==me->cell && guys[i].state==GS_THINKING && guys[i].type==GUY_GREEN)
		{
			guys[i].type=GUY_NONE;
			peopleCount--;
			Sound_Play(SND_CHOMP);
			Particle_Effect(PART_CHOMP,me->x,me->y-40.0f);
			return 1;
		}
	}
	return 0;
}

void KingFindDinner(guy_t *me)
{
	int x,y;

	x=me->cell%BLDG_WIDTH;
	y=me->cell/BLDG_WIDTH;

	// instajump to eat neighboring guys
	if(y>0)
	{
		me->cell-=BLDG_WIDTH;
		if(EatGuy(me))
		{
			me->x=(me->cell%BLDG_WIDTH)*BLDG_TILEX+BLDG_X+42.5f;
			me->y=(me->cell/BLDG_WIDTH)*BLDG_TILEY+BLDG_Y+80.0f;
			me->dx=0;
			me->dy=6;
			me->state=GS_MOVING;
			me->stepdy=-2;
			me->stepy=0;
			me->thinkDelay=1;
		}
		else
			me->cell+=BLDG_WIDTH;
	}
	if(y<BLDG_HEIGHT)
	{
		me->cell+=BLDG_WIDTH;
		if(EatGuy(me))
		{
			me->x=(me->cell%BLDG_WIDTH)*BLDG_TILEX+BLDG_X+42.5f;
			me->y=(me->cell/BLDG_WIDTH)*BLDG_TILEY+BLDG_Y+80.0f;
			me->dx=0;
			me->dy=-6;
			me->state=GS_MOVING;
			me->stepdy=-2;
			me->stepy=0;
			me->thinkDelay=1;
		}
		else
			me->cell-=BLDG_WIDTH;
	}
	if(x>0)
	{
		me->cell--;
		if(EatGuy(me))
		{
			me->x=(me->cell%BLDG_WIDTH)*BLDG_TILEX+BLDG_X+42.5f;
			me->y=(me->cell/BLDG_WIDTH)*BLDG_TILEY+BLDG_Y+80.0f;
			me->dx=6;
			me->dy=0;
			me->state=GS_MOVING;
			me->stepdy=-2;
			me->stepy=0;
			me->thinkDelay=1;
		}
		else
			me->cell++;
	}
	if(x<BLDG_WIDTH)
	{
		me->cell++;
		if(EatGuy(me))
		{
			me->x=(me->cell%BLDG_WIDTH)*BLDG_TILEX+BLDG_X+42.5f;
			me->y=(me->cell/BLDG_WIDTH)*BLDG_TILEY+BLDG_Y+80.0f;
			me->dx=-6;
			me->dy=0;
			me->state=GS_MOVING;
			me->stepdy=-2;
			me->stepy=0;
			me->thinkDelay=1;
		}
		else
			me->cell--;
	}
}

void Guy_Update(guy_t *me)
{
	if(me->animDelay)
		me->animDelay--;
	else
	{
		me->anim++;
		if(me->anim>7)
			me->anim=0;
		me->animDelay=4;
	}

	if(me->state==GS_THINKING)
	{
		if(me->king)
			KingFindDinner(me);
		if(me->thinkDelay)
			me->thinkDelay--;
		else
		{
			switch(me->type)
			{
				case GUY_GREEN:
					me->thinkDelay=levelDef[curLevel].personSpeed;
					if(bldg.lit[me->cell] && (rand()%4)==0)
						ChoosePlaceToGo(me);
					break;
				case GUY_ZOMBIE:
					me->thinkDelay=levelDef[curLevel].zombieSpeed;
					if((rand()%2)==0)
						ChoosePlaceToGo(me);
					break;
			}
		}
	}
	else if(me->state==GS_MOVING)
	{
		float tx,ty;

		me->x+=me->dx;
		me->y+=me->dy;
		me->stepy+=me->stepdy;
		me->stepdy+=0.5f;
		if(me->stepy>=0)
			me->stepdy=-3.0f;

		tx=(me->toCell%BLDG_WIDTH)*BLDG_TILEX+BLDG_X+42.5f;
		ty=(me->toCell/BLDG_WIDTH)*BLDG_TILEY+BLDG_Y+80.0f;

		if(fabs(me->x-tx)+fabs(me->y-ty)<4)
		{
			me->state=GS_THINKING;
			me->cell=me->toCell;
			me->dx=0;
			me->dy=0;
			me->stepy=0;
			me->stepdy=0;
			me->x=tx;
			me->y=ty;
			me->life=MAX_LIFE;
			if(me->type==GUY_ZOMBIE && (bldg.lit[me->cell]==0 || me->king))	// king can eat even in a lit room
			{
				EatGuy(me);
			}
		}
	}
	if(me->type==GUY_ZOMBIE && bldg.lit[me->cell])
	{
		Particle_Effect(PART_SMOKE,me->x+RandRange(-15.0f,15.0f),me->y-50.0f+RandRange(0,30.0f));
		Sound_Play(SND_SMOKE);
		if(me->state!=GS_MOVING)
		{
			ChoosePlaceToGo(me);
		}
		if(fabs(me->dx)+fabs(me->dy)<3)
		{
			me->dx*=3.0f;
			me->dy*=3.0f;
		}
		me->life--;
		if(me->life==0)
		{
			for(int i=0;i<30;i++)
				Particle_Effect(PART_SMOKE,me->x+RandRange(-15.0f,15.0f),me->y-50.0f+RandRange(0,30.0f));
			Sound_Play(SND_TORCHED);
			me->type=GUY_NONE;
			zombiesSlain++;
		}
	}
}

void Guys_Update(void)
{
	int i;

	for(i=0;i<MAX_GUYS;i++)
	{
		if(guys[i].type!=GUY_NONE)
			Guy_Update(&guys[i]);
	}
}

void Guys_Render(void)
{
	int sx,sy,i;
	byte animChart[8]={0,1,2,3,4,3,2,1};

	for(i=0;i<MAX_GUYS;i++)
	{
		if(guys[i].type!=GUY_NONE)
		{
			sx=animChart[guys[i].anim]*85;
			sy=0+85*(bldg.lit[guys[i].cell]==0)+(85*2*(guys[i].type==GUY_ZOMBIE)+85*2*(guys[i].king>0));
			guyPix->blitAlphaRect((float)sx,(float)sy,(float)sx+85,(float)sy+85,(short)(guys[i].x-GUY_CENTERX),(short)(guys[i].y-GUY_CENTERY-guys[i].stepy),false,false);
		}
	}
}