#include "menu.h"
#include "font.h"
#include "help.h"
#include "game.h"
#include "level.h"
#include "sound.h"

static KGraphic *titlePic,*cursorPic;
static int msx,msy;
static byte oldb,selected,lvlStart;

void Menu_Init(void)
{
	titlePic=new KGraphic;
	titlePic->loadPicture(KMiscTools::makeFilePath("gfx\\title.png"));
	cursorPic=new KGraphic;
	cursorPic->loadPicture(KMiscTools::makeFilePath("gfx\\cursor.png"));
	Font_Init();
	oldb=1;
	selected=0;
	lvlStart=0;
	while(curLevel>lvlStart+7)
		lvlStart+=8;
}

void Menu_Exit(void)
{
	Font_Exit();
	delete titlePic;
	delete cursorPic;
}

byte PointInRect(int x,int y,int rx,int ry,int rx2,int ry2)
{
	return (x>=rx && x<=rx2 && y>=ry && y<=ry2);
}

byte Menu_Update(void)
{
	byte b;
	int i;

	msx=KInput::getMouseX();
	msy=KInput::getMouseY();

	if(KInput::isPressed(K_VK_ESCAPE))
		return MENU_EXIT;

	b=KInput::getLeftButtonState();
	if(oldb==0 && b)
	{
		switch(selected)
		{
			case 0:
				break;
			case 1:
				Sound_Play(SND_FUSEPUT);
				Help();
				break;
			case 2:
				Sound_Play(SND_FUSEPUT);
				return MENU_PLAY;
				break;
			case 3:
				Sound_Play(SND_FUSEPUT);
				ToggleMusic();
				break;
			case 5:
				Sound_Play(SND_FUSEPUT);
				return MENU_EXIT;
				break;
			case 4:
				Sound_Play(SND_FUSEPUT);
				lvlStart+=8;
				if(lvlStart>=NUM_LEVELS)
					lvlStart=0;
				break;
			default:
				Sound_Play(SND_FUSEGET);
				if(levelAvailable[lvlStart+(selected-10)])
					curLevel=lvlStart+(selected-10);
				break;
		}
	}

	oldb=b;

	if(PointInRect(msx,msy,8,305,300,305+24))
		selected=1;
	else if(PointInRect(msx,msy,8,305+30,300,305+24+30))
		selected=2;
	else if(PointInRect(msx,msy,8,305+30*2,300,305+24+30*2))
		selected=3;
	else if(PointInRect(msx,msy,8,305+30*3,300,305+24+30*3))
		selected=5;
	else if(PointInRect(msx,msy,328,575,800,600))
		selected=4;
	else
	{
		selected=0;
		for(i=0;i<8;i++)
			if(PointInRect(msx,msy,328,330+i*30,800,330+i*30+24))
				selected=10+i;
	}


	return MENU_RUNNING;
}

void Menu_Render(void)
{
	int i;
	char s[38];

	glClear(GL_COLOR_BUFFER_BIT);
	KGraphic::setDefaultWorldView();

	titlePic->blitRect(0,0,800,303,0,0,false,false);

	if(selected==1)
		KGraphic::drawRect(8,312,300,322,1.0f,0.0f,0.0f,0.5f);
	Print(10,305,"INSTRUCTIONS");
	if(selected==2)
		KGraphic::drawRect(8,312+30,300,322+30,1.0f,0.0f,0.0f,0.5f);
	Print(10,335,"PLAY GAME");
	if(selected==3)
		KGraphic::drawRect(8,312+30*2,300,322+30*2,1.0f,0.0f,0.0f,0.5f);
	Print(10,365,"TOGGLE MUSIC");
	if(selected==5)
		KGraphic::drawRect(8,312+30*3,300,322+30*3,1.0f,0.0f,0.0f,0.5f);
	Print(10,395,"EXIT");

	Print(400,305,"LEVEL SELECT");
	if(selected==4)
		KGraphic::drawRect(328,575+7,800,575+17,1.0f,0.0f,0.0f,0.5f);
	Print(330,575,"MORE LEVELS");
	
	for(i=0;i<8;i++)
	{
		if(i+lvlStart<NUM_LEVELS)
		{
			if(selected==10+i)
				KGraphic::drawRect(328,(float)(330+i*30+7),800,(float)(330+i*30+17),0.5f,0.0f,0.0f+(0.5f*(curLevel==i+lvlStart)),1.0f);
			else if(curLevel==i+lvlStart)
				KGraphic::drawRect(328,(float)(330+i*30+7),800,(float)(330+i*30+17),0.0f,0.0f,0.5f,1.0f);
			sprintf(s,"%02d:%s",i+lvlStart+1,levelDef[i+lvlStart].name);
			if(levelAvailable[i+lvlStart])
				Print(330,330+i*30,s);
			else
				PrintFaint(330,330+i*30,s);
		}
	}

 	cursorPic->blitAlphaRect(0,0,23,45,msx-12,msy-8,false,false);

	KWindow::flipBackBuffer(true);
}

byte Menu_Run(void)
{
	dword now,start,interval,runs;
	byte retval;
	dword i;

	Menu_Init();

	start=KMiscTools::getMilliseconds();

	interval=0;
	while(1)
	{
		now=KMiscTools::getMilliseconds();
		interval+=now-start;
		start=now;
		if(interval>TIME_PER_FRAME*3)
			interval=TIME_PER_FRAME*3;
		runs=0;
		while(interval>=TIME_PER_FRAME)
		{
			runs++;
			interval-=TIME_PER_FRAME;
		}

		if(runs==0)
			runs=1;

		for(i=0;i<runs;i++)
		{
			retval=Menu_Update();
			if(retval!=MENU_RUNNING)
				i=runs;
		}
		if(retval!=MENU_RUNNING)
			break;
		Menu_Render();
	}
	Menu_Exit();
	return retval;
}