#include "particle.h"

KGraphic *chompPic,*smokePic,*sparkPic;
particle_t part[MAX_PARTICLES];

void Particle_Init(void)
{
	int i;

	for(i=0;i<MAX_PARTICLES;i++)
	{
		part[i].type=0;
	}
	chompPic=new KGraphic;
	chompPic->loadPicture(KMiscTools::makeFilePath("gfx\\chomp.png"));
	smokePic=new KGraphic;
	smokePic->loadPicture(KMiscTools::makeFilePath("gfx\\smokepuff.png"));
	sparkPic=new KGraphic;
	sparkPic->loadPicture(KMiscTools::makeFilePath("gfx\\spark_yellow.png"));
	sparkPic->setAlphaMode(false);
}

void Particle_Exit(void)
{
	delete chompPic;
	delete smokePic;
	delete sparkPic;
}

void Particle_Update(void)
{
	int i;

	for(i=0;i<MAX_PARTICLES;i++)
	{
		part[i].x+=part[i].dx;
		part[i].y+=part[i].dy;
		if(part[i].life)
			part[i].life--;
		else
			part[i].type=PART_NONE;
		
		switch(part[i].type)
		{
			case PART_CHOMP:
				part[i].scale+=0.02f;
				part[i].rot+=6.0f;
				if(part[i].rot>360.0f)
					part[i].rot-=360.0f;
				break;
			case PART_SMOKE:
				part[i].scale+=0.02f;
				part[i].rot+=12.0f;
				part[i].dy-=0.05f;

				if(part[i].blend>0.02f)
					part[i].blend-=0.02f;
				if(part[i].rot>360.0f)
					part[i].rot-=360.0f;
				break;
			case PART_SPARK:
				if(part[i].scale>0.01f)
					part[i].scale-=0.01f;
				part[i].rot+=(float)(rand()%40);
				if(part[i].blend>0.02f)
					part[i].blend-=0.02f;
				part[i].dy+=0.3f;
				if(part[i].rot>360.0f)
					part[i].rot-=360.0f;
				break;
		}
	}
}

void Particle_Render(void)
{
	int i;

	for(i=0;i<MAX_PARTICLES;i++)
	{
		switch(part[i].type)
		{
			case PART_CHOMP:
				chompPic->blitAlphaRectFx(0,0,96,96,(short)(part[i].x-(48.0f)),(short)(part[i].y-(48.0f)),part[i].rot,part[i].scale,1.0,false,false);
				break;
			case PART_SMOKE:
				smokePic->blitAlphaRectFx(0,0,16,16,(short)(part[i].x-(8.0f)),(short)(part[i].y-(8.0f)),part[i].rot,part[i].scale,part[i].blend,false,false);
				break;
			case PART_SPARK:
				sparkPic->blitAlphaRectFx(0,0,16,16,(short)(part[i].x-(8.0f)),(short)(part[i].y-(8.0f)),part[i].rot,part[i].scale,part[i].blend,false,false);
				break;
		}
	}
}

float RandRange(float min,float max)
{
	int i=rand();

	return min+((float)i*(max-min)/(float)RAND_MAX);
}

void Particle_Effect(byte type,float x,float y)
{
	int i,amt;

	if(type==PART_CHOMP || PART_SMOKE)
		amt=1;
	if(type==PART_SPARK)
		amt=8;

	for(i=0;i<MAX_PARTICLES;i++)
	{
		if(part[i].type==0)
		{
			amt--;
			part[i].type=type;
			part[i].x=x;
			part[i].y=y;
			switch(type)
			{
				case PART_CHOMP:
					part[i].dx=0;
					part[i].dy=0;
					part[i].scale=0.05f;
					part[i].rot=0;
					part[i].life=60;
					break;
				case PART_SPARK:
					part[i].dx=RandRange(-2.0f,2.0f);
					part[i].dy=RandRange(-6.0f,2.0f);
					part[i].scale=1.0f;
					part[i].rot=(float)(rand()%360);
					part[i].life=30+(rand()%80);
					part[i].scale=0.01f*(float)part[i].life;
					part[i].blend=0.02f*(float)part[i].life;
					if(part[i].blend>1.0f)
						part[i].blend=1.0f;
					break;
				case PART_SMOKE:
					part[i].dx=RandRange(-0.5f,0.5f);
					part[i].dy=RandRange(-0.5f,0.5f);
					part[i].scale=0.5f;
					part[i].rot=(float)(rand()%360);
					part[i].life=50+(rand()%40);
					part[i].blend=1.0f;
					break;
			}
			if(amt==0)
				return;
		}
	}
}
